//////////////////////////////////////////////////////////////////////////// 
// Signal_Processing_Supplementary_toolbox
//
// Copyright (c) by Y. K. Singh, CDAC, Kolkata.  2012
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
////////////////////////////////////////////////////////////////////////////

function [order,wn] = ellipord(wp,ws,rp,rs,opt)
    // inputs: wp,ws,rp, rs, opt
    //wp normalized passband edge frequency in radian per sample
    //ws normalized stopband edge frequency in radian per sample 
    //rp passband ripple in dB
    //rs stopband attenuation in dB
    //opt may be 's' for analog and 'z' for digital 
    //by default opt='z'
    //outputs: order, wn
    // order --elliptic filter order
    // wn -- elliptic natural frequency
    
[nargout,nargin]=argn(0);
if nargin == 4 then
    opt = 'z';
elseif nargin == 5 then
    if strcmp(opt,'z') & strcmp(opt,'s') then
        error('Invalid option for final argument.');
    end
end
msg=freqchk(wp,ws,opt);
if strcmp(msg,'') then
    error(msg);
end

ftype = 2*(length(wp) - 1);
if wp(1) < ws(1) then
    ftype = ftype + 1; // low (1) or reject (3)
else
    ftype = ftype + 2; // high (2) or pass (4)
end

// first, prewarp frequencies from digital (unit circle) to analog (imag. axis):
if ~strcmp(opt,'z') then // digital
    WP=tan(%pi*wp/2);
    WS=tan(%pi*ws/2);
else  // don't have to if analog already
    WP=wp;
    WS=ws;
end

// next, transform to low pass prototype with passband edge of 1 and stopband
// edges determined by the following: (see Rabiner and Gold, p.258)
if ftype == 1 then // low
    WA=WS/WP;
elseif ftype == 2 then // high
    WA=WP/WS;
elseif ftype == 3 then // stop
    fo = optimset();
    fo.Dsiplay='none';
    wp1 = lclfminbnd('bscost',WP(1),WS(1)-1e-12,fo,1,WP,WS,rs,rp,'ellip');
    WP(1) = wp1;
    wp2 = lclfminbnd('bscost',WS(2)+1e-12,WP(2),fo,2,WP,WS,rs,rp,'ellip');
    WP(2) = wp2;
    WA=(WS*(WP(1)-WP(2)))./(WS.^2 - WP(1)*WP(2));
elseif ftype == 4 then// pass
    WA=(WS.^2 - WP(1)*WP(2))./(WS*(WP(1)-WP(2)));
end

// find the minimum order elliptic filter to meet the more demanding spec:
WA = min(abs(WA));
epsilon = sqrt(10^(0.1*rp)-1);
k1 = epsilon/sqrt(10^(0.1*rs)-1);
k = 1/WA;
capk = ellipke([k^2 1-k^2]);
capk1 = ellipke([(k1^2) 1-(k1^2)]);
order = ceil(capk(1)*capk1(2)/(capk(2)*capk1(1)));

// if both warnings are in effect, only print the first one
if (1-k1.^2) == 1 then
   warning('(ellipord) attenuation too strenuous, estimated order is infinite.')
elseif k^2 == 1 then
   warning('(ellipord) band edges too close, estimated order is infinite.')
end

// natural frequencies are simply the passband edges (WP).
// finally, transform frequencies from analog to digital if necessary:
if ~strcmp(opt,'z') then // digital
    wn = wp;
else
    wn = WP(:)';
end
endfunction